@@ -3,7 +3,7 @@ Package for data processing in Python: https://github.com/WinVector/data_algebra
 """
 
 __docformat__ = "restructuredtext"
-__version__ = "1.6.4"
+__version__ = "1.6.5"
 
 __doc__ = """
 `data_algebra`<https://github.com/WinVector/data_algebra> is a piped data wrangling system
@@ -20,7 +20,7 @@ import data_algebra.data_model
 import data_algebra.data_ops
 # import for easy access for package users
 from data_algebra.data_ops import TableDescription, SQLNode, describe_table, descr, data, ex
-from data_algebra.expr_rep import lit, col
+from data_algebra.expr_rep import lit, col, d_, one
 import data_algebra.pandas_model
 
 data_algebra.pandas_model.register_pandas_model()
