@@ -2,9 +2,10 @@ import abc
 
 import data_algebra.data_ops
 import data_algebra.flow_text
+from data_algebra.shift_pipe_action import ShiftPipeAction
 
 
-class Arrow(abc.ABC):
+class Arrow(ShiftPipeAction):
     """
     Arrow from category theory: see Steve Awody,
     "Category Theory, 2nd Edition", Oxford Univ. Press, 2010 pg. 4.
@@ -13,7 +14,7 @@ class Arrow(abc.ABC):
     """
 
     def __init__(self):
-        pass
+        ShiftPipeAction.__init__(self)
 
     @abc.abstractmethod
     def dom(self):
@@ -23,55 +24,16 @@ class Arrow(abc.ABC):
     def cod(self):
         """return co-domain, object at head of arrow"""
 
-    @abc.abstractmethod
-    def apply_to(self, b):
-        """apply_to b, compose arrows (right to left)"""
-
     # noinspection PyPep8Naming
     @abc.abstractmethod
-    def act_on(self, X):
-        """act on X, must associate with composition"""
+    def act_on(self, b):
+        """act on b, must associate with composition"""
 
     # noinspection PyPep8Naming
     def transform(self, X):
         """transform X, may or may not associate with composition"""
         return self.act_on(X)
 
-    def __rshift__(self, other):  # override self >> other
-        return other.apply_to(self)
-
-    def __rrshift__(self, other):  # override other >> self
-        if isinstance(other, Arrow):
-            return self.apply_to(other)
-        return self.act_on(other)
-
-    # sklearn step style interface
-
-    # noinspection PyPep8Naming, PyUnusedLocal
-    def fit(self, X, y=None):
-        pass
-
-    # noinspection PyPep8Naming, PyUnusedLocal
-    def fit_transform(self, X, y=None):
-        self.fit(X, y=y)
-        return self.transform(X)
-
-    # noinspection PyUnusedLocal
-    @abc.abstractmethod
-    def get_feature_names(self, input_features=None):
-        """sklearn api"""
-
-    # noinspection PyUnusedLocal,PyMethodMayBeStatic
-    def get_params(self, deep=False):
-        return dict()
-
-    def set_params(self, **params):
-        pass
-
-    # noinspection PyPep8Naming
-    def inverse_transform(self, X):
-        raise TypeError("data_algebra does not support inverse_transform")
-
 
 class DataOpArrow(Arrow):
     """
@@ -80,17 +42,11 @@ class DataOpArrow(Arrow):
     """
 
     def __init__(
-        self, pipeline, *, free_table_key=None, forbidden_to_produce=None
+        self, pipeline, *, free_table_key=None
     ):
         assert isinstance(pipeline, data_algebra.data_ops.ViewRepresentation)
         self.pipeline = pipeline
-        self.strict = True
         t_used = pipeline.get_tables()
-        if forbidden_to_produce is None:
-            forbidden_to_produce = []
-        if isinstance(forbidden_to_produce, str):
-            forbidden_to_produce = [forbidden_to_produce]
-        self.forbidden_to_produce = forbidden_to_produce
         if free_table_key is None:
             if len(t_used) != 1:
                 raise ValueError(
@@ -104,74 +60,52 @@ class DataOpArrow(Arrow):
                 )
         self.free_table_key = free_table_key
         self.incoming_columns = list(t_used[free_table_key].column_names)
-        self.disallowed_columns = pipeline.forbidden_columns()[free_table_key]
         self.outgoing_columns = list(pipeline.column_names)
         self.outgoing_columns.sort()
         Arrow.__init__(self)
 
-    def get_feature_names(self, input_features=None):
+    def get_feature_names(self):
         cp = self.outgoing_columns.copy()
-        if (not self.strict) and (input_features is not None):
-            cp = cp + [f for f in input_features if f not in cp]
         return cp
 
-    def apply_to(self, b):
-        """replace self input table with b"""
+    def act_on(self, b, *, correct_ordered_first_call: bool = False):
+        """
+        Apply self onto b.
+        
+        :param b: item to act on, or item that has been sent to self.
+        :param correct_ordered_first_call: if True indicates this call is from __rshift__ or __rrshift__ and not the fallback paths.
+        """
+        assert isinstance(correct_ordered_first_call, bool)
         if isinstance(b, data_algebra.data_ops.ViewRepresentation):
             b = DataOpArrow(b)
-        assert isinstance(b, DataOpArrow)
-        # check categorical arrow composition conditions
-        missing = set(self.incoming_columns) - set(b.outgoing_columns)
-        if len(missing) > 0:
-            raise ValueError("missing required columns: " + str(missing))
-        problem_production = set(self.forbidden_columns()) - set(b.forbidden_to_produce)
-        if len(problem_production) > 0:
-            raise ValueError(
-                "did not document non-production of columns: " + str(problem_production)
-            )
-        excess = set(b.outgoing_columns) - set(self.incoming_columns)
-        if len(excess) > 0:
-            problem_excess = excess.intersection(self.forbidden_columns())
-            if len(problem_excess) > 0:
-                raise ValueError("forbidden incoming columns: " + str(excess))
-            if self.strict:
+        if isinstance(b, DataOpArrow):
+            # check categorical arrow composition conditions
+            missing = set(self.incoming_columns) - set(b.outgoing_columns)
+            if len(missing) > 0:
+                raise ValueError("missing required columns: " + str(missing))
+            excess = set(b.outgoing_columns) - set(self.incoming_columns)
+            if len(excess) > 0:
                 raise ValueError("extra incoming columns: " + str(excess))
-        new_pipeline = self.pipeline.replace_leaves({self.free_table_key: b.pipeline})
-        new_pipeline.get_tables()  # check tables are compatible
-        res = DataOpArrow(
-            pipeline=new_pipeline,
-            free_table_key=b.free_table_key,
-            forbidden_to_produce=self.forbidden_to_produce,
-        )
-        return res
-
-    # noinspection PyPep8Naming
-    def act_on(self, X):
+            new_pipeline = self.pipeline.replace_leaves({self.free_table_key: b.pipeline})
+            new_pipeline.get_tables()  # check tables are compatible
+            res = DataOpArrow(
+                pipeline=new_pipeline,
+                free_table_key=b.free_table_key,
+            )
+            return res
+        if correct_ordered_first_call and isinstance(b, ShiftPipeAction):
+            return b.act_on(self, correct_ordered_first_call=False)  # fall back
         # assume a pandas.DataFrame compatible object
         # noinspection PyUnresolvedReferences
-        cols = set(X.columns)
+        cols = set(b.columns)
         missing = set(self.incoming_columns) - cols
         if len(missing) > 0:
             raise ValueError("missing required columns: " + str(missing))
         excess = cols - set(self.incoming_columns)
-        if self.strict:
-            assert len(excess) == 0
+        assert len(excess) == 0
         if len(excess) > 0:
-            X = X[self.incoming_columns]
-        return self.pipeline.act_on(X)
-
-    # noinspection PyPep8Naming
-    def fit(self, X, y=None):
-        """Learn input and output types from example, and return self"""
-        # assume a pandas.DataFrame compatible object
-        self.act_on(X)
-        return self
-
-    # noinspection PyPep8Naming
-    def fit_transform(self, X, y=None):
-        """Learn input and output types from example, and return transform."""
-        out = self.transform(X)
-        return out
+            b = b[self.incoming_columns]
+        return self.pipeline.act_on(b)
 
     def dom(self):
         return DataOpArrow(
@@ -213,36 +147,24 @@ class DataOpArrow(Arrow):
     def required_columns(self):
         return self.incoming_columns.copy()
 
-    def forbidden_columns(self):
-        return self.disallowed_columns.copy()
-
     # noinspection PyMethodMayBeStatic
     def format_end_description(
-        self, *, required_cols, forbidden_cols, align_right=70, sep_width=2
+        self, *, required_cols, align_right=70, sep_width=2
     ):
         in_rep = [str(c) for c in required_cols]
         in_rep = data_algebra.flow_text.flow_text(
             in_rep, align_right=align_right, sep_width=sep_width
         )
         col_rep = [", ".join(line) for line in in_rep]
-        col_rep = " at least [ " + ",\n    ".join(col_rep) + " ]"
-        if (forbidden_cols is not None) and (len(forbidden_cols) > 0):
-            f_rep = [str(c) for c in forbidden_cols]
-            f_rep = data_algebra.flow_text.flow_text(
-                f_rep, align_right=align_right, sep_width=sep_width
-            )
-            f_rep = [", ".join(line) for line in f_rep]
-            col_rep = col_rep + " , and none of [ " + ",\n    ".join(f_rep) + " ]"
+        col_rep = " [ " + ",\n    ".join(col_rep) + " ]"
         return col_rep
 
     def __str__(self):
         in_rep = self.format_end_description(
             required_cols=self.incoming_columns,
-            forbidden_cols=self.disallowed_columns,
         )
         out_rep = self.format_end_description(
             required_cols=self.outgoing_columns,
-            forbidden_cols=self.forbidden_to_produce,
         )
         return (
             "[\n "
