@@ -34,9 +34,7 @@ def parse_assignments_in_context(*, ops, view):
         assert isinstance(k, str)
         orig_v = ops[k]  # make debugging easier
         v = orig_v
-        if isinstance(v, data_algebra.expr_rep.PreTerm):
-            v = v.replace_view(view)
-        else:
+        if not isinstance(v, data_algebra.expr_rep.PreTerm):
             if isinstance(v, str):
                 v = data_algebra.parse_by_lark.parse_by_lark(source_str=v, data_def=mp)
             else:
